/*
 * Sloggi - A Solaris Uname Jacket - http://www.roqe.org/sloggi
 * Copyright 2002 Konrad Rieck <kr@roqe.org> - All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright   
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright   
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR `AS IS'' AND ANY EXPRESS OR     
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "includes.h"
RCSID("$Id: refcnt.c,v 1.5 2002/09/04 21:12:31 kr Exp $");

/*
 * An implementation of a reference counter, in order to keep track 
 * who is calling the syscalls on smp machines. Thanks to Acpizer for
 * his code. 
 */

static kmutex_t refcnt_lock;
static int refcount;

void inc_refcnt()
{
    ASSERT(refcount >= 0);
    mutex_enter(&refcnt_lock);
    refcount++;
    mutex_exit(&refcnt_lock);
}

void dec_refcnt()
{
    mutex_enter(&refcnt_lock);
    refcount--;
    mutex_exit(&refcnt_lock);
    ASSERT(refcount >= 0);
}

void init_refcnt()
{
    mutex_init(&refcnt_lock, NULL, MUTEX_DEFAULT, NULL);

    mutex_enter(&refcnt_lock);
    refcount = 0;
    mutex_exit(&refcnt_lock);
}

void destroy_refcnt()
{
    mutex_destroy(&refcnt_lock);
}

boolean_t check_refcnt()
{
    mutex_enter(&refcnt_lock);
    if (refcount > 0) {
	mutex_exit(&refcnt_lock);
	return B_FALSE;
    }

    mutex_exit(&refcnt_lock);
    return B_TRUE;
}
