/*
 * Sloggi - A Solaris Uname Jacket - http://www.roqe.org/sloggi
 * Copyright 2002 Konrad Rieck <kr@roqe.org> - All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright   
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright   
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR `AS IS'' AND ANY EXPRESS OR     
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "includes.h"
RCSID("$Id: slogctl.c,v 1.2 2002/09/04 21:12:31 kr Exp $");

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <signal.h>
#include <unistd.h>

extern char *optarg;
extern int optind, opterr;

static struct utsname utsname;
static int enable = 0;

void print_usage()
{
    printf("Usage: slogctl [-hV] [-s sysname] [-r release] "
	   "[-v version] [-m machine]\n"
	   "Options:\n"
	   "   -s sysname   Set the name of the operating system.\n"
	   "                [Default: SunOS]\n"
	   "   -r release   Set the operating system release level.\n"
	   "                [Usually: 5.8, 5.9, ...]\n"
	   "   -v version   Set the the operating system version.\n"
	   "                [Default: Generic]\n"
	   "   -m machine   Set the machine hardware name (class).\n"
	   "                [Usually: sun4m, sun4u, i86pc, ...]\n"
	   "   -h           Print this help screen.\n"
	   "   -V           Print version information.\n"
	   "No options restore the original uname() setup\n");
}

void print_version()
{
    printf("Sloggi Control v%s, "
	   "Copyright 2002 Konrad Rieck <kr@roqe.org>\n", VERSION);
}

/*
 * Do the getopt stuff and have phun.
 */
void parse_options(int argc, char **argv)
{
    int c;

    /*
     * Parse commandline options.
     */
    while ((c = getopt(argc, argv, "s:r:m:v:hV")) != EOF) {

	if (optarg && strlen(optarg) > SYS_NMLN - 1) {
	    fprintf(stderr, "Error: slogctl: Argument too long.\n");
	    exit(EXIT_FAILURE);
	}

	switch (c) {
	case 's':
	    enable = 1;
	    bcopy(optarg, utsname.sysname, strlen(optarg) + 1);
	    break;
	case 'r':
	    enable = 1;
	    bcopy(optarg, utsname.release, strlen(optarg) + 1);
	    break;
	case 'v':
	    enable = 1;
	    bcopy(optarg, utsname.version, strlen(optarg) + 1);
	    break;
	case 'm':
	    enable = 1;
	    bcopy(optarg, utsname.machine, strlen(optarg) + 1);
	    break;
	case 'V':
	    print_version();
	    exit(EXIT_SUCCESS);
	    break;
	case 'h':
	default:
	    print_usage();
	    exit(EXIT_SUCCESS);
	}
    }
}

int main(int argc, char **argv)
{
    if (uname(&utsname) < 0) {
	perror("slogctl: read uname()");
	return EXIT_FAILURE;
    }

    parse_options(argc, argv);

    if (enable)
	bcopy(ENABLE_UTSNAME, utsname.nodename, strlen(ENABLE_UTSNAME) + 1);
    else
	bcopy(DISABLE_UTSNAME, utsname.nodename,
	      strlen(DISABLE_UTSNAME) + 1);

    if (uname(&utsname) < 0) {
	perror("slogctl: write uname()");
	return EXIT_FAILURE;
    }

    printf("%s %s %s %s %s\n",
	   utsname.sysname, utsname.nodename, utsname.release,
	   utsname.version, utsname.machine);

    return EXIT_SUCCESS;
}
