/*
 * Sloggi - A Solaris Uname Jacket - http://www.roqe.org/sloggi
 * Copyright 2002 Konrad Rieck <kr@roqe.org> - All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright   
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright   
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR `AS IS'' AND ANY EXPRESS OR     
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "includes.h"
RCSID("$Id: sloggi.c,v 1.1 2002/09/04 21:12:31 kr Exp $");

/*
 * This is the loadable module wrapper.
 */
extern struct mod_ops mod_miscops;

/*
 * Structure of the system-entry table.
 */
#ifdef _64BIT
extern struct sysent sysent32[];
#else
extern struct sysent sysent[];
#endif

/* 
 * Structure of the loaded modules
 */
extern struct modctl modules;

/*
 * Module linkage information for the kernel.
 */
static struct modlmisc modlmisc = {
    &mod_miscops,
    "Sloggi v" VERSION
};

struct modlinkage modlinkage = {
    MODREV_1,
    (void *) &modlmisc,
    NULL
};

/*
 * Syscall redirection vector table. 
 */
syscall_tab_t syscalls[] = {
    {0, (int64_t(*)())ujacket, SE_NOUNLOAD | SE_LOADED, SYS_uname,
     DFLT_ARG, 0},
    {0, 0, 0, 0, 0, 0}
};

#ifdef _64BIT
/*
 * Syscall redirection vector table. 
 */
syscall_tab_t syscalls32[] = {
    {0, (int64_t(*)())ujacket, SE_NOUNLOAD | SE_LOADED, SYS_uname,
     DFLT_ARG, 0},
    {0, 0, 0, 0, 0, 0}
};
#endif				/* _64BIT */

/*
 * Restore the old syscall vectors saved in the syscall struct.
 */
void restore_old_syscalls()
{
    int i;

    for (i = 0; syscalls[i].index; i++) {
	if (syscalls[i].index != SYS_UNUSED) {
	    dcmn_err((CE_CONT, "Restoring syscall %d.\n",
		      syscalls[i].index));
#ifdef _64BIT
	    sysent32[syscalls32[i].index].sy_callc = syscalls32[i].sc;
	    sysent32[syscalls32[i].index].sy_flags = syscalls32[i].flags;

	    if (syscalls32[i].narg != DFLT_ARG)
		sysent32[syscalls32[i].index].sy_narg = syscalls32[i].onarg;
#endif

	    sysent[syscalls[i].index].sy_callc = syscalls[i].sc;
	    sysent[syscalls[i].index].sy_flags = syscalls[i].flags;

	    if (syscalls[i].narg != DFLT_ARG)
		sysent[syscalls[i].index].sy_narg = syscalls[i].onarg;
	}
    }
}

/*
 * Store the old syscall vectors in the syscall struct.
 */
void store_old_syscalls()
{
    int i;

    for (i = 0; syscalls[i].index; i++) {
	if (syscalls[i].index != SYS_UNUSED) {
	    dcmn_err((CE_CONT, "Storing syscall %d.\n", syscalls[i].index));

#ifdef _64BIT
	    syscalls32[i].sc =
		(int64_t(*)())sysent32[syscalls32[i].index].sy_callc;
	    syscalls32[i].flags = sysent32[syscalls32[i].index].sy_flags;
	    syscalls32[i].onarg = sysent32[syscalls32[i].index].sy_narg;
#endif

	    syscalls[i].sc =
		(int64_t(*)())sysent[syscalls[i].index].sy_callc;
	    syscalls[i].flags = sysent[syscalls[i].index].sy_flags;
	    syscalls[i].onarg = sysent[syscalls[i].index].sy_narg;

	}
    }
}

/*
 * Set the new syscall vectors specified in the syscall struct.
 */
void set_new_syscalls()
{
    int i;

    for (i = 0; syscalls[i].index; i++) {
	if (syscalls[i].index != SYS_UNUSED) {
	    dcmn_err((CE_CONT, "Setting new syscall %d.\n",
		      syscalls[i].index));
#ifdef _64BIT

	    if (syscalls32[i].sc == syscalls[i].sc &&
		sysent32[syscalls32[i].index].sy_callc !=
		sysent[syscalls[i].index].sy_callc)
		cmn_err(CE_PANIC, "32 and 64 bit syscalls are not "
			"identical by default, but Sloggi claims they "
			"are!");

	    sysent32[syscalls32[i].index].sy_callc = syscalls32[i].nc;
	    sysent32[syscalls32[i].index].sy_flags = syscalls32[i].flags;

	    if (syscalls32[i].narg != DFLT_ARG)
		sysent32[syscalls32[i].index].sy_narg = syscalls32[i].narg;
#endif
	    sysent[syscalls[i].index].sy_callc = syscalls[i].nc;
	    sysent[syscalls[i].index].sy_flags = syscalls[i].flags;

	    if (syscalls[i].narg != DFLT_ARG)
		sysent[syscalls[i].index].sy_narg = syscalls[i].narg;
	}
    }
}

/*
 * Init and load the module.
 */
int _init()
{
    int i, cri;

    if ((i = mod_install(&modlinkage)) != 0) {
	cmn_err(CE_WARN, "Could not load module.");
	return i;
    } else {
	cmn_err(CE_CONT, "Sloggi v%s successfully loaded.\n", VERSION);
    }

    init_refcnt();
    init_ujacket();

    dcmn_err((CE_CONT, "Entering critical area.\n"));
    cri = ddi_enter_critical();

    /*
     * Redirect syscalls. 
     */
    store_old_syscalls();
    set_new_syscalls();

    ddi_exit_critical(cri);
    dcmn_err((CE_CONT, "Leaving critical area.\n"));

    return i;
}

int _info(struct modinfo *modinfop)
{
    return (mod_info(&modlinkage, modinfop));
}

/* 
 * "fini" and unload the module.
 */
int _fini()
{
    int i, cri;

    if (!check_refcnt())
	return set_errno(EBUSY);

    dcmn_err((CE_CONT, "Entering critical area.\n"));
    cri = ddi_enter_critical();

    /*
     * Restore old syscall vectors. 
     */
    restore_old_syscalls();

    ddi_exit_critical(cri);
    dcmn_err((CE_CONT, "Leaving critical area.\n"));

    destroy_ujacket();
    destroy_refcnt();

    if ((i = mod_remove(&modlinkage)) != 0)
	cmn_err(CE_WARN, "Could not unload module.");
    else
	cmn_err(CE_CONT, "Sloggi v%s successfully unloaded.\n", VERSION);

    return i;
}
