/*
 * Sloggi - A Solaris Uname Jacket - http://www.roqe.org/sloggi
 * Copyright 2002 Konrad Rieck <kr@roqe.org> - All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright   
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright   
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR `AS IS'' AND ANY EXPRESS OR     
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "includes.h"
RCSID("$Id: ujacket.c,v 1.4 2002/09/04 21:12:32 kr Exp $");

extern syscall_tab_t syscalls[];
#ifdef _64BIT
extern syscall_tab_t syscalls32[];
#endif

static krwlock_t utsname_rwlock;
static struct utsname utsname_fake;
static boolean_t active;

void utsname_copy(struct utsname *src, struct utsname *dst)
{
    bcopy(src->sysname, dst->sysname, strlen(src->sysname) + 1);
    bcopy(src->nodename, dst->nodename, strlen(src->nodename) + 1);
    bcopy(src->release, dst->release, strlen(src->release) + 1);
    bcopy(src->version, dst->version, strlen(src->version) + 1);
    bcopy(src->machine, dst->machine, strlen(src->machine) + 1);
}

int utsname_copyout(struct utsname *src, struct utsname *dst)
{
    if (copyout(src->sysname, dst->sysname, strlen(src->sysname) + 1))
	goto err;
    if (copyout(src->nodename, dst->nodename, strlen(src->nodename) + 1))
	goto err;
    if (copyout(src->release, dst->release, strlen(src->release) + 1))
	goto err;
    if (copyout(src->version, dst->version, strlen(src->version) + 1))
	goto err;
    if (copyout(src->machine, dst->machine, strlen(src->machine) + 1))
	goto err;
    return 0;
  err:
    return set_errno(EFAULT);
}

void init_ujacket()
{
    rw_init(&utsname_rwlock, NULL, RW_DRIVER, NULL);

    rw_enter(&utsname_rwlock, RW_WRITER);
    active = 0;
    utsname_copy(&utsname, &utsname_fake);
    rw_exit(&utsname_rwlock);
}

void destroy_ujacket()
{
    rw_destroy(&utsname_rwlock);
}

int64_t ujacket(struct utsname *name)
{
    int64_t ret;
    struct utsname utsname_tmp;

    inc_refcnt();
    dcmn_err((CE_CONT, "uname() syscall.\n"));


    if (!copyin(name, &utsname_tmp, sizeof(struct utsname))) {

	if (bcmp(utsname_tmp.nodename, ENABLE_UTSNAME,
		 strlen(ENABLE_UTSNAME)) == 0) {

	    if (CRED()->cr_ruid != 0) {
		dec_refcnt();
		return set_errno(EPERM);
	    }

	    cmn_err(CE_CONT, "Enabling fake uname().\n");
	    rw_enter(&utsname_rwlock, RW_WRITER);

	    active = 1;
	    bcopy(utsname.nodename, utsname_tmp.nodename,
		  strlen(utsname.nodename) + 1);
	    utsname_copy(&utsname_tmp, &utsname_fake);

	    rw_exit(&utsname_rwlock);
	}

	if (bcmp(utsname_tmp.nodename, DISABLE_UTSNAME,
		 strlen(DISABLE_UTSNAME)) == 0) {

	    if (CRED()->cr_ruid != 0) {
		dec_refcnt();
		return set_errno(EPERM);
	    }

	    cmn_err(CE_CONT, "Disabling fake uname().\n");
	    rw_enter(&utsname_rwlock, RW_WRITER);

	    active = 0;
	    utsname_copy(&utsname, &utsname_fake);

	    rw_exit(&utsname_rwlock);
	}
    }

    rw_enter(&utsname_rwlock, RW_READER);
    if (active) {
	ret = utsname_copyout(&utsname_fake, name);
    } else {
	ret = syscalls[UNAME].sc(name);
    }
    rw_exit(&utsname_rwlock);

    dec_refcnt();
    return ret;
}
